# Load the helper function and call it with arguments tailored for the testing
# grid.  It will make the morph configuration for us.  We share this function
# with the testing grid and have one fewer possible point of divergence.
import ../../lib/make-grid.nix {
  name = "Production";
  config = ./config.json;
  nodes = cfg:
    let
      sshUsers = import ./secrets/users.nix;

    # Get absolute vpn key directory path, as a string:
    monitoringvpnKeyDir = toString ./. + "/${cfg.monitoringvpnKeyDir}";

      # TBD: derive these automatically:
      hostsMap = {
        "172.23.23.1"  = [ "monitoring" "monitoring.monitoringvpn" ];
        "172.23.23.11" = [   "payments"   "payments.monitoringvpn" ];
        "172.23.23.21" = [ "storage001" "storage001.monitoringvpn" ];
        "172.23.23.22" = [ "storage002" "storage002.monitoringvpn" ];
        "172.23.23.23" = [ "storage003" "storage003.monitoringvpn" ];
        "172.23.23.24" = [ "storage004" "storage004.monitoringvpn" ];
        "172.23.23.25" = [ "storage005" "storage005.monitoringvpn" ];
      };
      vpnClientIPs = [
        "172.23.23.11"
        "172.23.23.21"
        "172.23.23.22"
        "172.23.23.23"
        "172.23.23.24"
        "172.23.23.25"
      ];
      nodeExporterTargets = [
        "monitoring"
        "payments"
        "storage001"
        "storage002"
        "storage003"
        "storage004"
        "storage005"
      ];

    in {
    # Here are the hosts that are in this morph network.  This is sort of like
    # a server manifest.  We try to keep as many of the specific details as
    # possible out of *this* file so that this file only grows as server count
    # grows.  If it grows too much, we can load servers by listing contents of
    # a directory or reading from another JSON file or some such.  For now,
    # I'm just manually maintaining these entries.
    #
    # The name on the left of the `=` is mostly irrelevant but it does provide
    # a default hostname for the server if the configuration on the right side
    # doesn't specify one.
    #
    # The names must be unique!
    "payments.privatestorage.io" = import ../../lib/make-issuer.nix (cfg // {
      publicIPv4 = "18.184.142.208";
      monitoringvpnIPv4 = "172.23.23.11";
      inherit monitoringvpnKeyDir;
      inherit sshUsers;
      hardware = ../../lib/issuer-aws.nix;
      stateVersion = "19.03";
    });

    "storage001" = import ../../lib/make-storage.nix (cfg // {
        cfg = import ./storage001-config.nix;
        inherit sshUsers;
        hardware = ./storage001-hardware.nix;
        stateVersion = "19.09";
        monitoringvpnIPv4 = "172.23.23.21";
        inherit monitoringvpnKeyDir;
    });
    "storage002" = import ../../lib/make-storage.nix (cfg // {
        cfg = import ./storage002-config.nix;
        inherit sshUsers;
        hardware = ./storage002-hardware.nix;
        stateVersion = "19.09";
        monitoringvpnIPv4 = "172.23.23.22";
        inherit monitoringvpnKeyDir;
    });
    "storage003" = import ../../lib/make-storage.nix (cfg // {
        cfg = import ./storage003-config.nix;
        inherit sshUsers;
        hardware = ./storage003-hardware.nix;
        stateVersion = "19.09";
        monitoringvpnIPv4 = "172.23.23.23";
        inherit monitoringvpnKeyDir;
    });
    "storage004" = import ../../lib/make-storage.nix (cfg // {
        cfg = import ./storage004-config.nix;
        inherit sshUsers;
        hardware = ./storage004-hardware.nix;
        stateVersion = "19.09";
        monitoringvpnIPv4 = "172.23.23.24";
        inherit monitoringvpnKeyDir;
    });
    "storage005" = import ../../lib/make-storage.nix (cfg // {
        cfg = import ./storage005-config.nix;
        inherit sshUsers;
        hardware = ./storage005-hardware.nix;
        stateVersion = "19.03";
        monitoringvpnIPv4 = "172.23.23.25";
        inherit monitoringvpnKeyDir;
    });

    "monitoring" = import ../../lib/make-monitoring.nix (cfg // {
      publicIPv4 = "monitoring.private.storage"; # XXX TBD when the machine is online
      monitoringvpnIPv4 = "172.23.23.1";
      inherit monitoringvpnKeyDir;
      inherit vpnClientIPs;
      inherit hostsMap;
      inherit nodeExporterTargets;
      hardware = ../../lib/issuer-aws.nix;
      stateVersion = "19.09";
      inherit sshUsers;
    });
  };
}
